/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// IMPORTING LOCAL PACKAGES
import 'package:zwap_design_system/atoms/atoms.dart';

class ScrollCardsProviderState<T> extends ChangeNotifier{

  final Map<DateTime, Map<T, bool>> elements;

  ScrollCardsProviderState({required this.elements}){
    DateTime firstDate = this.elements.keys.first;
    Map<T, bool> currentMapping = this.elements[firstDate]!;
    T firstElement = currentMapping.keys.first;
    this.elements[firstDate]![firstElement] = true;
  }

  void selectElement(DateTime day, T element){
    this.elements.forEach((key, value) {
      value.forEach((newKey, newValue) {
        this.elements[key]![newKey] = false;
      });
    });
    this.elements[day]![element] = true;
    notifyListeners();
  }
}


class ScrollCards<T> extends StatelessWidget{

  final Widget Function(T element, bool isSelected) getCard;

  ScrollCards({Key? key,
    required this.getCard,
  }): super(key: key);

  Widget _daySection(DateTime day, ScrollCardsProviderState<T> provider){
    List<T> mappingElements = provider.elements[day]!.keys.toList();
    DateTime now = DateTime.now();
    DateTime today = new DateTime(now.year, now.month, now.day);
    String title = today == day ? "Today" : "${day.weekday}, ${day.day} ${day.month}";
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 35),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: [
          Padding(
            padding: EdgeInsets.only(bottom: 40),
            child: ZwapText(
              textColor: ZwapColors.shades100,
              text: title,
              zwapTextType: ZwapTextType.h3,
            ),
          ),
          Padding(
            padding: EdgeInsets.only(top: 20),
            child: Column(
              children: List<Widget>.generate(mappingElements.length, ((index) =>
                  Padding(
                    padding: EdgeInsets.only(top: 10, bottom: 10, right: 20),
                    child: InkWell(
                      onTap: () => provider.selectElement(day, mappingElements[index]),
                      child: this.getCard(mappingElements[index], provider.elements[day]![mappingElements[index]]!),
                    ),
                  ))
              ),
            ),
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: ZwapVerticalScroll(
        child: Consumer<ScrollCardsProviderState<T>>(
          builder: (context, provider, child){
            List<DateTime> keysDateTime = provider.elements.keys.toList();
            return Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: List<Widget>.generate(keysDateTime.length, ((index) => this._daySection(keysDateTime[index], provider))),
            );
          }
        ),
      ),
    );
  }



}